from pyrogram import filters, enums
from pyrogram.handlers import MessageHandler
from pyrogram.errors.exceptions.bad_request_400 import MessageIdInvalid


class OnCmd:
    """
    ## Available Functions:
    
      ``on_cmd``: Main decorator
      ``on_cf``: Decorator to handle custom filters
    """
    @classmethod
    def on_cmd(
        self,
        command: list,
        group: int = 0,
        owner: bool = False,
        private: bool = True
    ):
        """
        ## Main decorator

        ### Arguments:

            ``command``: List of commands
            ``group`` (optional): Handler group (Defaults to 0)
            ``owner`` (optional): True if the command is only for owner (Defaults to False)
            ``private`` (optional): True if the command is only for private chats (Defaults to True)
        """
        if owner:
            filterm = (filters.user(self.OWNER_ID) & filters.me & filters.command(command) & ~filters.via_bot & ~filters.forwarded)
        else:
            filterm = (filters.me & filters.command(command) & ~filters.via_bot & ~filters.forwarded)
        def decorators(func):
            async def wrapper(client, message):
                chat_type = message.chat.type
                if private and chat_type != enums.ChatType.PRIVATE:
                    return await message.reply_text(text="`Perintah ini hanya untuk Pribadi!`")
                try:
                    await func(client, message)
                except MessageIdInvalid:
                    self.logs.warning("Jangan hapus pesan saat memproses. Ini dapat merusak bot!")
                except BaseException as e:
                    self.logs.error(e)
                    
            self.addhandler(filterm, wrapper, group)
            return wrapper
        return decorators 
    
    
    @classmethod
    def on_cf(self, custom_filters):
        """
        ## Decorator to handle custom filters

        ### Arguments:

            ``custom_filters``: Custom filters to handle
            ``handler_group`` (optional): Handler group (Defaults to 0)
        """
        def decorators(func):
            async def wrapper(client, message):
                try:
                    await func(client, message)
                except MessageIdInvalid:
                    self.logs.warning("Jangan hapus pesan saat memproses. Ini dapat merusak bot!")
                except BaseException as e:
                    self.logs.error(e)
                await message.continue_propagation()
            self.addhandler(custom_filters, wrapper, handler_group)
            return wrapper
        return decorators


    def addhandler(self, filterm, function, cmdgrp):
        """
        ## Add handler to the bot & userbot

        ### Arguments:

            ``function``: Callback function
            ``filterm``: Filters
            ``cmdgrp``: Command Group
        """
        self.add_handler(MessageHandler(function, filterm), cmdgrp)
