import os
import re
import base64
import random
import zipfile
import asyncio

from PIL import Image
from struct import pack
from datetime import datetime 
from pyrogram.file_id import FileId
from hachoir.parser import createParser
from hachoir.metadata import extractMetadata
from pymongo.errors import DuplicateKeyError
from marshmallow.exceptions import ValidationError


async def absolute_paths(directory):
    for dirpath, _, filenames in os.walk(directory):
        for f in filenames:
            yield os.path.abspath(os.path.join(dirpath, f))


def encode_file_id(s: bytes) -> str:
    r = b""
    n = 0

    for i in s + bytes([22]) + bytes([4]):
        if i == 0:
            n += 1
        else:
            if n:
                r += b"\x00" + bytes([n])
                n = 0

            r += bytes([i])

    return base64.urlsafe_b64encode(r).decode().rstrip("=")


class Getfile:
    def unpack_new_file_id(self, new_file_id):
        decoded = FileId.decode(new_file_id)
        file_id = encode_file_id(
            pack(
                "<iiqq",
                int(decoded.file_type),
                decoded.dc_id,
                decoded.media_id,
                decoded.access_hash,
            )
        )
        return file_id


    async def save_video(self, message, file_name, file_thumb, file_diamond):
        file_id = self.unpack_new_file_id(message.video.file_id)

        try:
            file = self.Media(
                file_id=file_id,
                file_name=file_name,
                file_thumb=file_thumb,
                file_diamond=file_diamond,
            )
        except ValidationError:
            self.logs.exception('Terjadi error saat menyimpan file di database')
        else:
            try:
                await file.commit()
            except DuplicateKeyError:
                await message.reply_text(file_name + " sudah tersimpan di database")
                self.logs.warning(file_name + " sudah tersimpan di database")
                await asyncio.sleep(2)
                await message.delete()
            else:
                await message.reply_text(file_name + " disimpan dalam database")
                self.logs.info(file_name + " disimpan dalam database")
                await asyncio.sleep(2)
                await message.delete()
               
               
    async def save_video2(self, message, file_name, file_durr, file_size, file_thumb):
        file_id = self.unpack_new_file_id(message.video.file_id)

        try:
            file = self.Media2(
                file_id=file_id,
                file_name=file_name,
                file_durr=file_durr,
                file_size=file_size,
                file_thumb=file_thumb,
            )
        except ValidationError:
            self.logs.exception('Terjadi error saat menyimpan file di database')
        else:
            try:
                await file.commit()
            except DuplicateKeyError:
                self.logs.warning(file_name + " sudah tersimpan di database")
            else:
                self.logs.info(file_name + " disimpan dalam database")


    async def get_search(self, query):
        if not query:
            raw_pattern = '.'
        elif ' ' not in query:
            raw_pattern = r'(\b|[\.\+\-_])' + query + r'(\b|[\.\+\-_])'
        else:
            raw_pattern = query.replace(' ', r'.*[\s\.\+\-_\(\)\[\]]')
        try:
            regex = re.compile(raw_pattern, flags=re.IGNORECASE)
        except:
            return [], ''
        filter = {'file_name': regex}
        total_results = await self.Media.count_documents(filter)
        cursor = self.Media.find(filter)
        cursor.sort('$natural', -1)
        files = await cursor.limit(total_results).to_list(length=total_results)
        return files


    async def get_search2(self, query, offset=0):
        if not query:
            raw_pattern = '.'
        elif ' ' not in query:
            raw_pattern = r'(\b|[\.\+\-_])' + query + r'(\b|[\.\+\-_])'
        else:
            raw_pattern = query.replace(' ', r'.*[\s\.\+\-_\(\)\[\]]')
        try:
            regex = re.compile(raw_pattern, flags=re.IGNORECASE)
        except:
            return [], ''
        filter = {'file_name': regex}
        total_results = await self.Media2.count_documents(filter)
        next_offset = offset + 10
        if next_offset > total_results:
            next_offset = ''
        cursor = self.Media2.find(filter)
        cursor.sort('$natural', -1)
        cursor.skip(offset).limit(10)
        files = await cursor.to_list(length=10)
        return files, next_offset


    async def unzip(self, file):
        dirname = "resource/unzip/"
        with zipfile.ZipFile(file, "r") as zip_ref:
            zip_ref.extractall(dirname)
        os.remove(file)
        extracted_files = [i async for i in absolute_paths(dirname)]
        return extracted_files
        
        
    async def cut_video(self, video_file, start_time, end_time):
        # https://stackoverflow.com/a/13891070/4723940
        out_put_file_name = os.path.dirname(video_file) + str(datetime.now()) + ".mp4"
        file_generator_command = [
            "ffmpeg",
            "-i",
            video_file,
            "-ss",
            str(start_time),
            "-to",
            str(end_time),
            "-async",
            "1",
            "-strict",
            "-2",
            out_put_file_name
        ]
        process = await asyncio.create_subprocess_exec(
            *file_generator_command,
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
        )
        stdout, stderr = await process.communicate()
        stderr.decode().strip()
        stdout.decode().strip()
        if os.path.lexists(out_put_file_name):
            return out_put_file_name
        else:
            return None


    async def take_screen_shot(self, video_file, output_directory, ttl):
        # https://stackoverflow.com/a/13891070/4723940
        out_put_file_name = output_directory + "/" + str(datetime.now()) + ".jpg"
        file_genertor_command = [
            "ffmpeg",
            "-ss",
            str(ttl),
            "-i",
            video_file,
            "-vframes",
            "1",
            out_put_file_name,
        ]
        # width = "90"
        process = await asyncio.create_subprocess_exec(
            *file_genertor_command,
            # stdout must a pipe to be accessible as process.stdout
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
        )
        # Wait for the subprocess to finish
        stdout, stderr = await process.communicate()
        stderr.decode().strip()
        stdout.decode().strip()
        if os.path.lexists(out_put_file_name):
            return out_put_file_name
        else:
            return None


    async def getvideo(self, files):
        width = 0
        height = 0
        duration = 0
        if str(files).split(".")[-1] in ["mkv", "m4v", "MP4", "mov", "webm", "mpe4", "mpeg", "MOV"]:
            path = str(files).split(".")[0] + ".mp4"
            os.rename(files, path) 
            files = str(files).split(".")[0] + ".mp4"
        thumb_image = str(self.id) + ".jpg"
        self.logs.info(files)
        metadata = extractMetadata(createParser(files))
        if metadata.has("duration"):
            duration = metadata.get("duration").seconds
        if not os.path.exists(thumb_image):
            thumb_image = await self.take_screen_shot(
                files, os.path.dirname(files),
                random.randint(0, duration - 1),
            )
        self.logs.info(thumb_image)
        metadata = extractMetadata(createParser(thumb_image))
        if metadata.has("width"):
            width = metadata.get("width")
        if metadata.has("height"):
            height = metadata.get("height")
        Image.open(thumb_image).convert("RGB").save(thumb_image)
        image = Image.open(thumb_image)
        image.resize((90, height))
        image.save(thumb_image, "JPEG")
            
        return files, thumb_image, width, height, duration
